/*
 * *****************************************************************
 * *                                                               *
 * *    Copyright (c) Digital Equipment Corporation, 1991, 1995    *
 * *                                                               *
 * *   All Rights Reserved.  Unpublished rights  reserved  under   *
 * *   the copyright laws of the United States.                    *
 * *                                                               *
 * *   The software contained on this media  is  proprietary  to   *
 * *   and  embodies  the  confidential  technology  of  Digital   *
 * *   Equipment Corporation.  Possession, use,  duplication  or   *
 * *   dissemination of the software and media is authorized only  *
 * *   pursuant to a valid written license from Digital Equipment  *
 * *   Corporation.                                                *
 * *                                                               *
 * *   RESTRICTED RIGHTS LEGEND   Use, duplication, or disclosure  *
 * *   by the U.S. Government is subject to restrictions  as  set  *
 * *   forth in Subparagraph (c)(1)(ii)  of  DFARS  252.227-7013,  *
 * *   or  in  FAR 52.227-19, as applicable.                       *
 * *                                                               *
 * *****************************************************************
 */
/*
 * HISTORY
 */
/*
 * @(#)$RCSfile: cma_tis.h,v $ $Revision: 1.1.5.4 $ (DEC) $Date: 1994/05/31 20:27:08 $
 */
/*
 *  FACILITY:
 *
 *	Thread-Independent Services, a subsidiary of DECthreads
 *
 *  ABSTRACT:
 *
 *	External definitions for CMA TIS services
 *
 *  AUTHORS:
 *
 *	Webb Scales
 *
 *  CREATION DATE:
 * 
 *	29 April 1992
 *
 *  MODIFIED BY:
 *
 *	Webb Scales
 *	Dave Butenhof
 *
 *  LAST MODIFIED:
 *
 *	26 May 1994
 */
#ifndef CMA_TIS_INCLUDE
#define CMA_TIS_INCLUDE


/*
 * The following routines comprise the DECthreads "Thread Independent Services".
 * These routines provide the indicated functionality when used in a program
 * or application in which threads are present.  In the absence of threads,
 * these functions return immediately, imposing the minimum possible overhead
 * on their caller.
 *
 * TIS objects (e.g., mutexes) can be used in the same program or application
 * which uses DECthreads objects; however, TIS objects are referenced by
 * address instead while DECthreads objects are referenced by handle.  TIS 
 * objects, therefore, may not be interoperable (i.e., interchangeable) with 
 * DECthreads objects.
 */


/*
 * TIS configuration symbols
 */
#define _CMA_TIS__OSF	1
#define _CMA_TIS__VMS	2

#ifndef _CMA_TIS_OS_
# if defined(__OSF__) || defined(__osf__)
#  define   _CMA_TIS_OS_    _CMA_TIS__OSF
# endif
# if defined(vms) || defined(__vms) || defined(VMS) || defined(__VMS) || defined(__vms__)
#  define   _CMA_TIS_OS_    _CMA_TIS__VMS
# endif
#endif


/*
 * TIS data types.
 */

#if defined(__STDC__) || defined(VAXC) || defined(__mips__)
# define _TIS_PROTO_(proto) proto
  typedef void	*cma_tis_addr_t;
#else
# define _TIS_PROTO_(proto) ()
  typedef char	*cma_tis_addr_t;
#endif

/*
 * TIS condition variable: the address of an internal condition variable object.
 */
typedef cma_tis_addr_t	cma_tis_cond_t;

/*
 * TIS mutex: the address of an internal mutex object.
 */
typedef cma_tis_addr_t	cma_tis_mutex_t;

/*
 * TIS one-time initialization control block and its initialization macro.
 *
 * Declare a one time initialization control block as:
 *
 *      static cma_tis_once_t	block = cma_tis_once_init;
 */
typedef struct CMA_TIS_ONCE_T {
    long	field1;
    long	field2;
    long	field3;
    }				cma_tis_once_t;

#define cma_tis_once_init	{0, 0, 0}

/*
 * Per-thread context types
 */
typedef long cma_tis_key_t;
typedef void (*cma_tis_destructor_t) _TIS_PROTO_ ((cma_tis_addr_t value));

/*
 * TIS thread-ID: the address of a internal thread object.
 */
typedef cma_tis_addr_t	cma_tis_thread_t;


#if _CMA_TIS_OS_ == _CMA_TIS__VMS
  /*
   * On VMS, these routines must begin with "cma$" not "cma_", so use a 
   * macro to translate on those platforms.
   */
# define cma_tis_errno_get_addr		cma$tis_errno_get_addr
# define cma_tis_errno_set_value	cma$tis_errno_set_value
# define cma_tis_vmserrno_get_addr	cma$tis_vmserrno_get_addr
# define cma_tis_vmserrno_set_value	cma$tis_vmserrno_set_value
# define cma_tis_cond_create		cma$tis_cond_create
# define cma_tis_cond_delete		cma$tis_cond_delete
# define cma_tis_cond_broadcast		cma$tis_cond_broadcast
# define cma_tis_cond_signal		cma$tis_cond_signal
# define cma_tis_cond_wait		cma$tis_cond_wait
# define cma_tis_key_create		cma$tis_key_create 
# define cma_tis_key_get_context	cma$tis_key_get_context 
# define cma_tis_key_set_context	cma$tis_key_set_context 
# define cma_tis_lock_global		cma$tis_lock_global 
# define cma_tis_mutex_create		cma$tis_mutex_create
# define cma_tis_mutex_delete		cma$tis_mutex_delete
# define cma_tis_mutex_lock		cma$tis_mutex_lock
# define cma_tis_mutex_trylock		cma$tis_mutex_trylock
# define cma_tis_mutex_unlock		cma$tis_mutex_unlock
# define cma_tis_once			cma$tis_once 
# define cma_tis_thread_get_self	cma$tis_thread_get_self
# define cma_tis_unlock_global		cma$tis_unlock_global 
#endif

/*
 * TIS functions.
 */

/*
 * Get the address of errno 
 */
extern int *cma_tis_errno_get_addr _TIS_PROTO_ ((void));

/*
 * Set all known errno cells to a particular value
 */
extern void cma_tis_errno_set_value _TIS_PROTO_ ((int value));

#if _CMA_TIS_OS_ == _CMA_TIS__VMS
/*
 * Get the address of the VMS-specific errno
 */
extern int *cma_tis_vmserrno_get_addr _TIS_PROTO_ ((void));

/*
 * Set all known VMS-specific errno cells to a particular value
 */
extern void cma_tis_vmserrno_set_value _TIS_PROTO_ ((int value));
#endif

/*
 * Create a TIS condition variable.
 *
 * "cond" is a pointer to an abstract condition variable, passed by reference, 
 * which receives the address of the new condition variable object.
 */
extern int cma_tis_cond_create _TIS_PROTO_ ((cma_tis_cond_t *cond));

/*
 * Delete a TIS condition variable
 *
 * "cond" is the address of the abstract condition variable, passed by 
 * reference, which is to be deleted.
 */
extern int cma_tis_cond_delete _TIS_PROTO_ ((cma_tis_cond_t *cond));

/*
 * Broadcast on a TIS condition variable
 *
 * "cond" is the address of the abstract condition variable, passed by 
 * reference, on which to broadcast.
 */
extern int cma_tis_cond_broadcast _TIS_PROTO_ ((cma_tis_cond_t *cond));

/*
 * Signal a TIS condition variable
 *
 * "cond" is the address of the abstract condition variable, passed by 
 * reference, on which to signal.
 */
extern int cma_tis_cond_signal _TIS_PROTO_ ((cma_tis_cond_t *cond));

/*
 * Wait on a TIS condition variable
 *
 * "cond" is the address of the abstract condition variable, passed by 
 * reference, on which to wait.
 */
extern int cma_tis_cond_wait _TIS_PROTO_ ((cma_tis_cond_t *cond, 
					   cma_tis_mutex_t *mutex));

/*
 * Lock and unlock the DECthreads global lock (if present).
 */
extern int cma_tis_lock_global _TIS_PROTO_ ((void));
extern int cma_tis_unlock_global _TIS_PROTO_ ((void));

/*
 * Create a per-thread context (also known as "thread-specific storage" key).
 *
 * "Key" is the address of a cell which will receive the key value; this
 * value is used in calls to cma_tis_key_set_context() and 
 * cma_tis_key_get_context() to get and set the per-thread context associated
 * with this key.  The same key value is used in each threads to access that
 * thread's context.
 * "Destructor" is the address of a routine which is called to destroy
 * (e.g., deallocate) the per-thread context value when the thread terminates.
 */
extern int cma_tis_key_create _TIS_PROTO_ ((cma_tis_key_t *key, 
					    cma_tis_destructor_t destructor));

/*
 * Get the per-thread context value associated with a particular key.
 *
 * "Key" is a key value returned by a call to cma_tis_key_create().
 * "Value" is the address of a cell which will receive the per-thread context
 * value associated with this key.  The value returned is dependent on the
 * value of the key specified, but unaffected by the actions of any other
 * threads.
 */
extern int cma_tis_key_get_context _TIS_PROTO_ ((cma_tis_key_t key, 
					    cma_tis_addr_t *value));

/*
 * Set the per-thread context value associated with a particular key.
 *
 * "Key" is a key value returned by a call to cma_tis_key_create().
 * "Value" is value to be associated with this key.  Once set, this value 
 * can be retrieved in this thread using the same key in a call to 
 * cma_tis_key_get_context(); however, other threads' use of this key value
 * is unaffected.
 */
extern int cma_tis_key_set_context _TIS_PROTO_ ((cma_tis_key_t key, 
					    cma_tis_addr_t value));

/*
 * Create a TIS mutex
 *
 * "mutex" is a pointer to an abstract mutex, passed by reference, which 
 * receives the address of the new mutex object.
 */
extern int cma_tis_mutex_create _TIS_PROTO_ ((cma_tis_mutex_t *mutex));

/*
 * Delete a TIS mutex
 *
 * "mutex" is the address of the abstract mutex, passed by reference, which is
 * to be deleted.
 */
extern int cma_tis_mutex_delete _TIS_PROTO_ ((cma_tis_mutex_t *mutex));

/*
 * Lock a TIS mutex
 *
 * "mutex" is the address of the abstract mutex, passed by reference, which is
 * to be locked.
 */
extern int cma_tis_mutex_lock _TIS_PROTO_ ((cma_tis_mutex_t *mutex));

/*
 * Non-blocking TIS mutex lock
 *
 * "mutex" is the address of the abstract mutex, passed by reference, which is
 * to be locked.
 *
 * The function returns a boolean value indicating if the mutex was 
 * successfully locked.
 */
extern int cma_tis_mutex_trylock _TIS_PROTO_ ((cma_tis_mutex_t *mutex));

/*
 * Unlock a TIS mutex
 *
 * "mutex" is the address of the abstract mutex, passed by reference, which is
 * to be unlocked.
 */
extern int cma_tis_mutex_unlock _TIS_PROTO_ ((cma_tis_mutex_t *mutex));

/*
 * Execute a one-time initialization routine.
 *
 * The specified routine is called exactly once during a program's execution,
 * even if multiple threads call cma_tis_once() simultaneously.  This is 
 * useful for performing one-time initialization tasks (e.g., for a facility
 * API), such as creating mutexes and per-thread-context keys.
 *
 * "init_block" is the address of a one-time initialization control block.  This
 * should be in static storage and unique to the initialization routine.
 * "init_routine" is the address of the initialization routine to call.
 * "arg" is a user argument which is passed to the initialization routine.
 */
extern int cma_tis_once _TIS_PROTO_ ((
	cma_tis_once_t	*init_block, 
	void		(*init_rtn)(cma_tis_addr_t arg), 
	cma_tis_addr_t	init_arg));

/*
 * Get current thread "ID".  (Returns zero if there are no threads.)
 */
extern cma_tis_thread_t cma_tis_thread_get_self _TIS_PROTO_ ((void));
#endif
