/*
 * *****************************************************************
 * *                                                               *
 * *    Copyright (c) Digital Equipment Corporation, 1991, 1995    *
 * *                                                               *
 * *   All Rights Reserved.  Unpublished rights  reserved  under   *
 * *   the copyright laws of the United States.                    *
 * *                                                               *
 * *   The software contained on this media  is  proprietary  to   *
 * *   and  embodies  the  confidential  technology  of  Digital   *
 * *   Equipment Corporation.  Possession, use,  duplication  or   *
 * *   dissemination of the software and media is authorized only  *
 * *   pursuant to a valid written license from Digital Equipment  *
 * *   Corporation.                                                *
 * *                                                               *
 * *   RESTRICTED RIGHTS LEGEND   Use, duplication, or disclosure  *
 * *   by the U.S. Government is subject to restrictions  as  set  *
 * *   forth in Subparagraph (c)(1)(ii)  of  DFARS  252.227-7013,  *
 * *   or  in  FAR 52.227-19, as applicable.                       *
 * *                                                               *
 * *****************************************************************
 */
/*
 * HISTORY
 */
/* @(#)src/common/lib/libvxvm/volclient.h	5.9 16 Mar 1993 01:44:16 - Copyright (c) 1993 VERITAS Software Corp. */
#ident	"@(#)vxvm:common/lib/libvxvm/volclient.h	5.9"

/*
 * Copyright(C)1993 VERITAS Software Corporation.  ALL RIGHTS RESERVED.
 * UNPUBLISHED -- RIGHTS RESERVED UNDER THE COPYRIGHT
 * LAWS OF THE UNITED STATES.  USE OF A COPYRIGHT NOTICE
 * IS PRECAUTIONARY ONLY AND DOES NOT IMPLY PUBLICATION
 * OR DISCLOSURE.
 * 
 * THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND
 * TRADE SECRETS OF VERITAS SOFTWARE.  USE, DISCLOSURE,
 * OR REPRODUCTION IS PROHIBITED WITHOUT THE PRIOR
 * EXPRESS WRITTEN PERMISSION OF VERITAS SOFTWARE.
 * 
 *               RESTRICTED RIGHTS LEGEND
 * USE, DUPLICATION, OR DISCLOSURE BY THE GOVERNMENT IS
 * SUBJECT TO RESTRICTIONS AS SET FORTH IN SUBPARAGRAPH
 * (C) (1) (ii) OF THE RIGHTS IN TECHNICAL DATA AND
 * COMPUTER SOFTWARE CLAUSE AT DFARS 252.227-7013.
 *               VERITAS SOFTWARE
 * 4800 GREAT AMERICA PARKWAY, SANTA CLARA, CA 95054
 */

#ifndef _VOLCLIENT_H
#define _VOLCLIENT_H

/*
 * volclient.h - volume manager client side header file
 *
 * This file contains data specific to the client side of the volume
 * manager library interface.
 */

#include <sys/vol.h>
#include <volipc.h>

/* standard exit codes for client utilities */
#define VEX_OK		0		/* no error to report */
#define VEX_USAGE	1		/* usage of utility is incorrect */
#define VEX_SYNTAX	2		/* syntax err in cmd or description */
#define VEX_NOVOLD	3		/* volume daemon not running */
#define VEX_IPC		4		/* communications error with daemon */
#define VEX_OSERR	5		/* unexpected system call error */
#define VEX_LOST	6		/* commit status lost */
#define VEX_UTILERR	7		/* error in utility */
#define VEX_TIMEOUT	8		/* transaction timed out */
#define VEX_NODG	9		/* disk group not located */
#define VEX_CHANGED	10		/* unexpected change to database */
#define VEX_NOENT	11		/* named object is not in database */
#define VEX_EXIST	12		/* named object already in database */
#define VEX_BUSY	13		/* object is in use */
#define VEX_NOUSETYPE	14		/* no default usage type */
#define VEX_BADUSETYPE	15		/* named usage type not valid */
#define VEX_ASSOC	16		/* object is associated */
#define VEX_DISASSOC	17		/* object is not associated */
#define VEX_LAST	18		/* object is last associated */
#define VEX_TOOMANY	19		/* too many associations */
#define VEX_INVAL	20		/* operation does not make sense */
#define VEX_IOERR	21		/* I/O error */
#define VEX_NOPLEX	22		/* A volume has no plexes */
#define VEX_NOSUBDISK	23		/* A plex has no subdisks */
#define VEX_UNSTARTABLE	24		/* volume is unstartable */
#define VEX_STARTED	25		/* volume is already started */
#define VEX_UNSTARTED	26		/* volume is not started */
#define VEX_DETACHED	27		/* volume or plex is detached */
#define VEX_DISABLED	28		/* volume or plex is disabled */
#define VEX_ENABLED	29		/* volume or plex is enabled */
#define VEX_ASSERT	30		/* unrecognized error */
#define VEX_OPEN	31		/* device open or mounted */

#define VEX_UNKNOWN	7		/* unrecognized error - compatibility */

/* directory that contains usage type utilities */
#ifndef VOL_UT_DIR
#define VOL_UT_DIR	"/etc/vol/type"
#endif

#ifndef VOL_ADMIN_DIR
#define VOL_ADMIN_DIR	"/sbin"
#endif

/* maximum possible length for a path to a usage type utility */
#define VOL_UT_LEN	(sizeof(VOL_UT_DIR) + NAME_LEN + \
			 sizeof("/*verylongutilityname"))*/

/*
 * structure and defines for the vol_copy function
 */

struct vol_copy {
	ulong_t cp_flag;		/* flags affecting operation */
	int cp_nsrc;			/* number of source plexes */
	int cp_ndst;			/* number of destination plexes */
	int cp_nerr;			/* number of plexes with errors */
	int cp_nsrcerr;			/* number of src plexes with errors */
	char **cp_src;			/* names of source plexes */
	char **cp_dst;			/* names of destination plexes */
	struct plx_ent *cp_err;		/* plexes with errors */
	voff_t cp_start;		/* start of region to copy (bytes) */
	voff_t cp_len;			/* length of region to copy */
	voff_t cp_err_offset;		/* offset of an I/O error */
	size_t cp_size;			/* bytes to copy in one ioctl */
	int cp_iodelay;			/* delay to introduce before each I/O */
#if defined(__STDC__)
	int (*cp_err_handler)(int, struct vol_copy *); /* called on error */
#else	/* not __STDC__ */
	int (*cp_err_handler)();	/* handler to call on error */
#endif	/* __STDC__ */
	VOID *cp_private;		/* private pointer unused by vol_copy */
};

/* flags stored in vol_copy.cp_flag */
#define VCP_REPORT_EXACT	0x00000001	/* report exact block numbers */
#define VCP_SLOW		0x00000002	/* introduce a delay per I/O */

/* externally visible library functions */
#if defined(__STDC__)

extern int vol_check(void);
extern int vol_open(void);
extern void vol_set_rendezvous(const char *);
extern void vol_set_retry(int, int);
extern void vol_close(void);
extern int vol_info(int *, pid_t *, struct vol_id *);
extern struct vol_id *vol_id(void);
extern int vol_trans(const struct vol_lock *, int, unsigned, ulong_t, ulong_t);
extern int vol_commit(ulong_t, ulong_t);
extern int vol_abort();
extern int sd_creat(const char *, const struct sdrec *);
extern int plex_creat(const char *, const struct plexrec *);
extern int vol_creat(const char *, const struct volrec *);
extern int sd_delete(const char *);
extern int plex_delete(const char *);
extern int vol_delete(const char *);
extern int sd_as(const char *, const char *, voff_t);
extern int sd_as_log(const char *, const char *);
extern int plex_as(const char *, const char *);
extern int sd_dis(const char *);
extern int plex_dis(const char *);
extern int sd_change(const char *, const struct sdrec *, ulong_t);
extern int plex_change(const char *, const struct plexrec *, ulong_t);
extern int vol_change(const char *, const struct volrec *, ulong_t);
extern int vol_summary(struct vol_summary *);
extern int vol_name(volname_t *, vol_object_t, int);
extern int sd_get(volname_t *, struct sdrec *, int);
extern int plex_get(volname_t *, struct plexrec *, int);
extern int vol_get(volname_t *, struct volrec *, int);
extern int vol_get_plex(const char *, struct plexrec *, int);
extern int plex_get_sd(const char *, struct sdrec *, int);
extern int sd_search(const char *, struct sdrec *, int);
extern int plex_search(const char *, struct plexrec *, int);
extern int vol_search(const char *, struct volrec *, int);
extern int volmgr_search(const char *, struct sdrec *, int *,
		struct plexrec *, int *, struct volrec *, int *);
extern char *vol_pattern(const char *);
extern int volmgr_unique(char *name);
extern int vol_diag(int, const char *, int, char *, int);
extern int vol_copy(int, struct vol_copy *);
extern char *vol_copy_plexerror(int);
extern char *vol_copy_volerror(int);
extern struct dg_config *dg_getconfig(ulong_t);
extern struct dg_config *dg_getconfig_name(const char *, ulong_t);
extern struct dg_config *dg_getconfig_id(const char *, ulong_t);
extern struct dg_config *dg_getconfig_diskdev(const char *, ulong_t);
extern struct dg_config_vec *dg_getconfig_all(ulong_t);
extern struct dg_config_diff *dg_getconfig_diff(const char *,
				volseqno_t, ulong_t);
extern struct dg_config *dg_getconfig_search(const char *, ulong_t);
extern int da_creat(const char *, const struct darec *);
extern int da_change(const char *, const struct darec *, ulong_t);
extern int da_delete(const char *);
extern int da_get(volname_t *, struct darec *, int);
extern int da_find(char *, const char *);
extern int dm_creat(const char *, const struct dmrec *);
extern int dm_change(const char *, const struct dmrec *, ulong_t);
extern int dm_get(volname_t *, struct dmrec *, int);
extern int dm_as_da(const char *, const char *);
extern int dm_dis_da(const char *);
extern int dg_creat(const char *, const struct dgrec *);
extern int dg_change(const struct dgrec *, ulong_t);
extern int dg_get(struct dgrec *);
extern int dg_get_import_id(volrid_t, struct dgrec *);
extern int dg_set_current(const char *);
extern int dg_set_current_id(const char *);
extern struct dg_free *dg_get_free(volname_t *, size_t);
extern struct dg_locate *dg_locate(size_t, char **);
extern int dg_import(const char *, const char *, const char *, ulong_t);
extern int dg_deport(const char *);
extern int dg_deport_id(const char *);
extern int dg_flush_name(const char *);
extern int dg_flush_id(const char *);
extern struct dg_misc_info *dg_get_info_name(const char *);
extern struct dg_misc_info *dg_get_info_id(const char *);
extern int vxvm_make_name(const char *, const char *, size_t, size_t, char *);
extern int vxvm_rename(const char *, const char *);
extern int vxvm_delete(const char *);
extern int vxvm_delete_rid(volrid_t);
extern struct vxvm_record *vxvm_get(const char *, ulong_t);
extern struct vxvm_record *vxvm_get_rid(volrid_t, ulong_t);
extern struct vxvm_record *vxvm_get_hier(const char *, ulong_t);
extern struct vxvm_record *vxvm_get_hier_rid(volrid_t, ulong_t);

#else	/* not __STDC__ */

extern int vol_check();
extern int vol_open();
extern void vol_set_rendezvous();
extern void vol_set_retry();
extern void vol_close();
extern int vol_info();
extern struct vol_id *vol_id();
extern int vol_trans();
extern int vol_commit();
extern int vol_abort();
extern int sd_creat();
extern int plex_creat();
extern int vol_creat();
extern int sd_delete();
extern int plex_delete();
extern int vol_delete();
extern int sd_as();
extern int sd_as_log();
extern int plex_as();
extern int sd_dis();
extern int plex_dis();
extern int sd_change();
extern int plex_change();
extern int vol_change();
extern int vol_summary();
extern int vol_name();
extern int sd_get();
extern int plex_get();
extern int vol_get();
extern int vol_get_plex();
extern int plex_get_sd();
extern int sd_search();
extern int plex_search();
extern int vol_search();
extern int volmgr_search();
extern char *vol_pattern();
extern int volmgr_unique();
extern int vol_copy();
extern char *vol_copy_plexerror();
extern char *vol_copy_volerror();
extern struct dg_config *dg_getconfig();
extern struct dg_config *dg_getconfig_name();
extern struct dg_config *dg_getconfig_id();
extern struct dg_config *dg_getconfig_disk();
extern struct dg_config_vec *dg_getconfig_all();
extern struct dg_config_diff *dg_getconfig_diff();
extern struct dg_config *dg_getconfig_search();
extern int da_creat();
extern int da_change();
extern int da_delete();
extern int da_get();
extern int da_find();
extern int dm_creat();
extern int dm_change();
extern int dm_get();
extern int dm_as_da();
extern int dm_dis_da();
extern int dg_creat();
extern int dg_change();
extern int dg_get();
extern int dg_get_import_id();
extern int dg_set_current();
extern int dg_set_current_id();
extern struct dg_free *dg_get_free();
extern int dg_import();
extern int dg_deport();
extern int dg_deport_id();
extern int dg_flush_name();
extern int dg_flush_id();
extern struct dg_misc_info *dg_get_info_name();
extern struct dg_misc_info *dg_get_info_id();
extern int vxvm_make_name();
extern int vxvm_rename();
extern int vxvm_delete();
extern int vxvm_delete_rid();
extern struct vxvm_record *vxvm_get();
extern struct vxvm_record *vxvm_get_rid();
extern struct vxvm_record *vxvm_get_hier();
extern struct vxvm_record *vxvm_get_hier_rid();

#endif	/* __STD__ */

/* externally visible variables defined by the library */
extern int vol_errno;		/* last volume manager error */
extern char *vol_errlist[];	/* strings for vol_errno values */
extern int vol_nerr;		/* number of strings in vol_errlist */
extern char *vol_errstr;	/* error string, set by some functions */
extern volop_t vol_err_volop;	/* operation associated with an error */
extern int vol_err_errno2;	/* auxiliary error number */
extern char *vol_err_name1;	/* name of a record associated with error */
extern char *vol_err_name2;	/* name of a second record */
extern ulong_t vold_flags;	/* flags passed from vold on connection */
extern long vold_version;	/* version number of vold */

#endif /* _VOLCLIENT_H */
